% function [P, X] = orthofact(obs, mn)
% obs: the observation matrix
% mn:  the mean of the observations.
% Compute orthographic sfm.  If mean, mn, is given use it,
% Otherwise calculate it.
%
% (c) Neil Birkbeck 2010
function [P, X] = orthofact(obs, mn, rect)

if (nargin < 2) || isempty(mn),
  mn = mean(obs, 2);
end

if nargin < 3,
  rect = true;
end


mn_obs = obs - repmat(mn, [1, size(obs, 2)]);

[U,S,V] = svd(mn_obs);

S = diag(S);
S = diag(S(1:3));

P = U(:,1:3)*sqrt(S);
X = sqrt(S)*V(:, 1:3)';


if rect
  % Compute the rectifying transform
  qp = lsqnonlin(@f1,[1,0,0,1,0,1],[],[],[],P);
  qp = chol(sym(qp))';
  
  % Apply the rectifying transform.
  P = P*(qp);
  X = inv(qp)*X;
end

% Might as well make the points 4D
P(:,4) = mn;
X(4,:) = 1;


% Extract the symmetric matrix given by the 6 components.
function m=sym(X)
m=[X(1) X(2) X(3);
   X(2) X(4) X(5);
   X(3) X(5) X(6)];
return;

%
% Function to optimize the rotational adjustment that happens to
% the camera vectors.
%
%  rp are the camera vectors.
function F=f1(X,rp)
if size(X,1)==3,
  m = X;
else
  m=sym(X);
end
F=zeros(size(rp,1)/2,3);
for i=1:(size(rp,1)/2),
  ci=rp(i,:);
  cj=rp(size(rp,1)/2+i,:);
  F(i,1)=ci*m*ci'-1;
  F(i,2)=cj*m*cj'-1;
  F(i,3)=ci*m*cj';
end
F=F(:);
return;
