% Plot some results for the regularized svd for image data.
%
% images = zeros(32, 32, 40);
% for i=1:40,
%  [im,map,alpha]=imread(sprintf('/tmp/%04d.png', i));
%   images(:,:,i) = im(:,:,1)
% end
% 
% (c) Neil Birkbeck, 2010
function plotspheres(X);

if 0,
figure(1);
reconstruct_and_plot(X, rand(size(X)) > 0.1, 0.6, '90 % visible');

figure(2);
reconstruct_and_plot(X, rand(size(X)) > 0.3, 0.6, '70 % visible');

figure(3);
reconstruct_and_plot(X, rand(size(X)) > 0.5, 0.4, '60 % visible');
end
% Simulate the occlusion, some random numbers.
P = (X>=0.6) | X==X(1,1) | (rand(size(X)) > 0.7*(0.6 - X)/0.6);
figure(4);
reconstruct_and_plot(X, P, 3.0, 'Occ');



% Search for a lambda such that the regularization gives the
% desired rank, start with the given lambda
function Z2=find_reconstruction(X, P, lambda, desired_rank)
l = lambda
while 1
  Z2 = spectreg(X, P, [l], 1e-12, X.*P);
    
  fprintf('rank: %d\n', rank(Z2{1}));
  
  if rank(Z2{1}) <= desired_rank,
    break;
  end  
  l = l + 0.01;
end
return;


% Reconstruct the values using regularized PCA and plot the results.
function reconstruct_and_plot(X, P, lambda, tit)
mn = sum(X.*P, 2)./max(sum(P, 2), 1);
mn_full = repmat(mn, [1, size(X, 2)]);

Z2 = find_reconstruction(X - mn_full, P, lambda, 4);

fprintf('----------------------------------\nTitle: %s\n', tit);

% Show all images.
for i=1:size(X, 2),
  subplot(3, 3, 1);
  imagesc(reshape(X(:,i).*P(:,i), [32, 32]));colormap gray;
  title('Input');
  
  subplot(3, 3, 2);
  imagesc(reshape(X(:,i).*P(:,i) + mn, [32, 32]));colormap gray;
  title('Reconstructed mean');
  
  subplot(3, 3, 3);
  imagesc(reshape(Z2{1}(:,i) + mn, [32, 32]));colormap gray;
  title('Reconstructed reg SVD');
  
  pause(0.1);
  drawnow;
end

Xr_mean = X.*P + mn_full.*(~P);
Xr_mean = Xr_mean - mn_full;

% Which mean should we subtract here?
Xr_recon = X.*P + (Z2{1} + mn_full).*(~P);
Xr_recon = Xr_recon - mn_full;

[B_mean, s_mean] = pca_basis(Xr_mean, 1:4);
[B_recon, s_recon] = pca_basis(Xr_recon, 1:4);

subplot(3, 3, 5);
imagesc(reshape(B_mean(:,1), [32, 32]));
subplot(3, 3, 8);
imagesc(reshape(B_mean(:,2), [32, 32]));

subplot(3, 3, 6);
imagesc(reshape(B_recon(:,1), [32, 32]));
subplot(3, 3, 9);
imagesc(reshape(B_recon(:,2), [32, 32]));

'Mean:' 
s_mean(1:4)/sum(s_mean(:))

'Recon:'
s_recon(1:4)/sum(s_recon(:))


% 
function [B, sd] = pca_basis(X, keep)
[U, D] = eig(X*X');
dd = diag(D);
[sd,I] = sort(-dd);
B = U(:,I(keep));
sd = -sd;



