%
% Implementation of the "spectral regularization algorithms for
% learning large incomplete matrices"
%
% In the end, I tended to only pass in a single lambda and do the
% lambda logic selection outside of this function.
%
% Not optimized however, they must do something special.
% (c) Neil Birkbeck, 2010
%
function Zs=spectreg(Y, P, lambdas, epsilon, init, do_hard)

if nargin < 4
  epsilon = 8e-5;
end

if nargin < 5
  init = Y;
end

if nargin < 6,
  do_hard = false;
end

% Choose the correct type.
type = 'svd_proj_soft';

if do_hard
  type = 'svd_proj_hard';
end


Zs = {};

for l=1:length(lambdas),
  Zs{l} = solve_for_lambda(Y, P, lambdas(l), epsilon, type ,init);
%  Zs{l} = solve_for_lambda(Y, P, lambdas(l), epsilon, 'svd_proj_hard');
end


function [Z] = solve_for_lambda(X, P, lambda, epsilon, svd_proj, init)
Zold = init;

max_its=1000;
norm_type = 'fro';

for i=1:max_its,
  Znew = feval(svd_proj, P.*X + (~P).*Zold, lambda);;

  err = (norm(Znew - Zold, norm_type)^2/norm(Zold, norm_type)^2);
  if mod(i, 300)==0
    fprintf('%d/%d, %e\n', i, max_its, err);
  end
  
  if err < epsilon
    break
  end
  
  Zold = Znew;
end

Z = Znew;


function Z=svd_proj_soft(Y, lambda)
[U, S, V] = svd(Y, 'econ');
Z = U * (max(S - lambda, 0)) * V';

function Z=svd_proj_hard(Y, lambda)
[U, S, V] = svd(Y,'econ');
Z = U * (S.* (S>=lambda)) * V';
